;------------------------------------------------------------------------------
;   rac^dug presents a real dycp for the dycp's sake alone
;      ĺ          since it was actually unnecessary because one could
;  ļ   use the many hardware sprites as well, but who cares
; the weird world of rac                                             rac^dug 2k
;------------------------------------------------------------------------------

.org $0000

.org $0040			; ----- interrupt - vertical blank
	ld	a,($c105)
	ld	($ff43),a	; reset screen scrolling
	reti

.org $0048			; ----- interrupt - lcdc status
	call	dycp
	reti

.org $0050			; ----- interrupt - timer overflow
	reti

.org $0058			; ----- interrupt - serial transfer completion
	reti

.org $0060			; ----- interrupt - hi-to-lo of p10-p13
	reti

.org $0100			; ----- execution entry point
	jp	main

.org $0104			; ----- standard nintendo logo necessary

.db $CE,$ED,$66,$66,$CC,$0D,$00,$0B,$03,$73,$00,$83,$00,$0C,$00,$0D
.db $00,$08,$11,$1F,$88,$89,$00,$0E,$DC,$CC,$6E,$E6,$DD,$DD,$D9,$99
.db $BB,$BB,$67,$63,$6E,$0E,$EC,$CC,$DD,$DC,$99,$9F,$BB,$B9,$33,$3e

.db "da ding         "		; 16 bytes cartridge name
.db "sd"			; new licensee code
.db $00				; gb-sgb indicator set to gameboy
.db $00				; cartridge type set to rom Only
.db $00				; rom size set to 256kbit=32kbyte
.db $00				; ram size set to zero
.db $00				; destination code set to japanese
.db $a4				; old licensee code set to konami
.db $01				; rom version set to 1
.db $DA				; complement check
.db $ff,$ff			; checksum to be corrected later


; ***
; *** the main program starts here
; ***

.org $0150

main
	di                     	; disable interrupts
	ld	a,%00000011	; lcdc and vertical blank interrupts only
	ld	($ffff),a
	sub	a		; misc standard init things
	ld	($ff42),a	; screen scroll y = 0
	ld	($ff43),a	; screen scroll x = 0

	call	waitvbl		; must be in vbl before turning the screen off
	ld	a,%00010001	; lcd controller = off (no picture on screen)
				; windowbank = $9800 (not used)
				; window     = off
				; bg chr     = $8000
				; bg bank    = $9800
				; obj        = 8*8
				; obj        = off
				; bg         = on
	ld	($ff40),a
	call	waitvbl		; must be in vbl before turning the screen on
	ld	a,%10010001	; lcd controller = on
	ld	($ff40),a

	ld	hl,$CFFF	; initialize stack
	ld	sp,hl

        ld	a,%01000100	; lyc equal coincidence
	ld	($ff41),a	; set lcdc interrupt mode

        ld	a,$40		; on rasterline $4d
	ld	($ff45),a	; cause lcdc interrupt

	call	nor_col		; normal palette
	call	move_char	; move charset to $8000
	call	move_text	; move text to $9800

        call	waitvbl		; must be in vbl before turning screen on
        ld	a,%10010001	; lcd controller on

	ld	hl,dycptext	; do some setup stuff for the dycp routine
	ld	a,l
	ld	($C100),a
	ld	a,h
	ld	($C101),a
	ld	hl,positiontab
	ld	a,l
	ld	($C102),a
	ld	a,h
	ld	($C103),a
	sub	a
	ld	($c104),a
	ld	($c105),a
	ld	($c106),a
	ld	($c107),a
	ei			; enable interrupts

mainloop
	jr	mainloop	; jr mainloop, do nothing except interrupts

waitvbl				; ----- wait for vbl subroutine
	ld	a,($ff40)
	add	a,a
	ret	nc
notyet
	ld	a,($ff44)	; $ff44=LCDC Y-Pos
	cp	$90		; $90 and bigger = in VBL
	jr	nz,notyet	; loop until it $90
	ret

nor_col				; ----- sets the colors to normal palette
	ld	a,%11100100	; grey 3=11 (black)
				; grey 2=10 (dark grey)
				; grey 1=01 (ligth grey)
				; grey 0=00 (transparent)
	ld	($ff47),a
	ret

move_char			; ----- copy the charset
	ld	hl,$8000
	ld	bc,charset
	ld	d,$00		; like move 248 bytes man
	ld	e,$08
lp1
	ld	a,(bc)
	ldi	(hl),a
	ldi	(hl),a          ; store the same byte again
	inc	bc
	dec	d
	jp	nz,lp1
	dec	e
	jp	nz,lp1
	ret

move_text			; ----- copy the tilemap
	ld	hl,$9800
	ld	bc,dycpmap
	ld	d,$00
	ld	e,$04		; one whole background map Screen
wloop1
	ld	a,(bc)
	ld	(hli),a
	inc	bc
	dec	d
	jp	nz,wloop1
	dec	e
	jp	nz,wloop1
	ret

dycp                            ; ----- dypc charset copy routine
	sub	a
	ld	($ff43),a	; reset screen scrolling
	ld   	hl,$8000	; now delete the first 168 tiles
	ld   	d,$54
deleteloop			; a is still zero
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a
	ldi	(hl),a		; delete all 16 bytes of two chars
	dec	d
	jr	nz,deleteloop
outerloop			; outer loop 21 colums, counted in d which is 0
	ld	a,($c100)	; now get the character to yield, first we get
	ld	l,a		; the table's base adress
	ld	a,($c101)
	ld	h,a
	ld	c,d		; we want the d-th character
	ld	b,$00

	add	hl,bc		; now we address the d-th character
	
	ld	a,($c106)	; now get the scroll string offset
	ld	c,a
	ld	a,($c107)
	ld	b,a

	add	hl,bc

	ld	a,(hl)		; and save it in a
	ld	b,$00		; now we transfer it to bc
	ld	c,a		; and multiply by 8 to get
	sla	c		; the correct tiledata offset
	rl	b		; (one character equals 8 bytes)
	sla	c
	rl	b
	sla	c
	rl	b
	ld	hl,charset
	add	hl,bc		; now hl points to the absolute tiledata
	push	hl		; we save that address to the stack
	ld	a,($c102)	; now we read the y-position from table,
	ld	l,a		; first we get the table's base address
	ld	a,($c103)
	ld	h,a
	ld	b,$00
	ld	c,d		; we want the d-th table entry
	add	hl,bc		; hl now points to the absolute table entry, now
                                ; we want to add the temporal displacement
	ld	a,($c104)
	ld	b,$00
	ld	c,a
	add	hl,bc
 
	ld	a,(hl)		; relevant y-position is now in a
	ld	b,$00
	ld	c,a		; we transfer it to bc
	sla	c		; and multiply by 2 (1 char row equals 2 bytes)
	rl	b
	push	bc		; we save the relevant y-position to stack
	ld	hl,$8000	; start adress tile 0 in hl
	ld	b,$00		; colum number to b
	ld	c,d
	sla	c		; we multiply by 128
	rl	b
	sla	c
	rl 	b
	sla	c
	rl	b
	sla	c
	rl	b
	sla	c
	rl	b
	sla	c
	rl	b
	sla	c
	rl	b
	add	hl,bc		; hl now points to top of column in charset
	pop	bc		; we rebuild the relevant y-position
	add	hl,bc		; now we add y position
	pop	bc		; rebiuld character starting position
	push	de		; save d for outer loop counter
	ld	d,$08		; copy 8 bytes, one complete character
innerloop
	ld	a,(bc)
	ldi	(hl),a
	ldi	(hl),a		; again, write the very same row
	inc	bc
	dec	d
	jr	nz,innerloop
	pop	de
	inc	d
	ld	a,d
	cp	$15		; have we reached the last column ?
	jr	nz,outerloop
	ld	a,($c104)	; increment sinus counter
	inc	a
	ld	($c104),a

	ld	a,($c105)	; increment screen scroll offset
	inc	a
	and	%00000111
	ld	($c105),a
	cp	%00000111

	jp	nz,keepoffset	; need we change the string offset ?

	ld	a,($c106)	; yes, so we do it
	ld	l,a
	ld	a,($c107)
	ld	h,a
	inc	hl
	ld	a,l
	ld	($c106),a
	ld	a,h
	and	%00000001
	ld	($c107),a

keepoffset
	ret			; we have finished all necessary updates

; Charset ripped from standard vga mode $13 font

charset				; ----- here goes the charset
#include "font.inc"

positiontab			; ----- here goes the dycp table
#include "sintab.asm"

dycpmap				; ----- this is the static image map
;--- row 01
.db $00,$08,$10,$18,$20,$28,$30,$38,$40,$48,$50,$58,$60,$68,$70,$78,$80,$88,$90,$98,$a0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 02
.db $01,$09,$11,$19,$21,$29,$31,$39,$41,$49,$51,$59,$61,$69,$71,$79,$81,$89,$91,$99,$a1
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 03
.db $02,$0a,$12,$1a,$22,$2a,$32,$3a,$42,$4a,$52,$5a,$62,$6a,$72,$7a,$82,$8a,$92,$9a,$a2
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 04
.db $03,$0b,$13,$1b,$23,$2b,$33,$3b,$43,$4b,$53,$5b,$63,$6b,$73,$7b,$83,$8b,$93,$9b,$a3
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 05
.db $04,$0c,$14,$1c,$24,$2c,$34,$3c,$44,$4c,$54,$5c,$64,$6c,$74,$7c,$84,$8c,$94,$9c,$a4
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 06
.db $05,$0d,$15,$1d,$25,$2d,$35,$3d,$45,$4d,$55,$5d,$65,$6d,$75,$7d,$85,$8d,$95,$9d,$a5
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 07
.db $06,$0e,$16,$1e,$26,$2e,$36,$3e,$46,$4e,$56,$5e,$66,$6e,$76,$7e,$86,$8e,$96,$9e,$a6
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- row 08
.db $07,$0f,$17,$1f,$27,$2f,$37,$3f,$47,$4f,$57,$5f,$67,$6f,$77,$7f,$87,$8f,$97,$9f,$a7
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00
;--- dummy
.db $b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0,$b0
.db $00,$00,$00,$00,$00,$00,$00,$00,$00,$00,$00

dycptext			; ----- here is the dycp text
.db "                    "
.db "          Hallo Arne"
.db ", hier ist also mein"
.db " Gameboy-Gefunze. Ic"
.db "h habe da einen alte"
.db "n Demo-Effekt aus C6"
.db "4-Zeiten ausgegraben"
.db " und auf dem Gameboy"
.db " implementiert (ich "
.db "bin brigens nicht d"
.db "er erste). Falls Du "
.db "irgendwelchen Gamebo"
.db "y-Stuff haben willst"
.db ", kannst Du mich ja "
.db "mal anmailen. Bis de"
.db "nne Florian.        "
.db "                    "
.db "                    "
.db "                    "
.db "Hey, Du bist ja imme"
.db "r noch da! Tja, selb"
.db "er Schuld, hast wohl"
.db " kein Leben, was ?  "
.db "                    "
.db "                    "
.db "                    "
.db "                    "

;Variablen:
;
;(c100) dycp-string adresse lo-byte
;(c101) dycp-string adresse hi-byte
;(c102) positiontab adresse lo-byte
;(c103) positiontab adresse hi-byte
;(c104) position in der positiontab
;(c105) screen scroll offset
;(c106) dycp-string-offset adresse lo-byte
;(c107) dycp-string-pffset adresse hi-byte

.block $8000-$			; pad to $8000 with zeros
.end
